"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceBits = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const util_1 = require("util");
const chalk_1 = require("chalk");
const globby_1 = __importDefault(require("globby"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const readFile = util_1.promisify(fs_1.readFile);
const writeFile = util_1.promisify(fs_1.writeFile);
/* istanbul ignore next */
/**
 * Replaces markdown bits in Markdown files that live within `docsRoot`.
 * Searches for bits that match the format `${bit-name}` with content
 * from `${options.bitsDirName}/bit-name.md` within the same folder.
 * @param docsRoot defaults to `"./documents"`.
 * @param options
 */
async function replaceBits(docsRoot = './documents', { bitsDirName = '_bits', outputDir = './dist/documents' } = {}) {
    await mkdirp_1.default(outputDir);
    const bitPathsGlob = `${docsRoot}/**/${bitsDirName}/*.md`;
    const bits = await loadBits();
    return _replaceBits();
    async function loadBits() {
        const bits = new Map();
        const bitPaths = await globby_1.default(bitPathsGlob);
        await Promise.all(bitPaths.map(async (bitPath) => {
            bits.set(bitPath.toString(), (await readFile(bitPath)).toString());
        }));
        return bits;
    }
    async function _replaceBits() {
        const docPaths = await globby_1.default([
            `${docsRoot}/**/*.md`,
            `!${bitPathsGlob}`,
        ]);
        return Promise.all(docPaths.map(replaceBit));
        async function replaceBit(docPath) {
            const bitPattern = /\${([\w\d-]+)}/g;
            const docDir = path_1.dirname(docPath);
            const contents = (await readFile(docPath)).toString();
            const outputPath = path_1.resolve(outputDir, path_1.relative(docsRoot, docPath));
            await mkdirp_1.default(path_1.dirname(outputPath));
            return writeFile(outputPath, contents.replace(bitPattern, (_m, bitName) => {
                const absoluteDocsRoot = path_1.resolve(docsRoot);
                let currentDir = docDir;
                while (true) {
                    const bitPath = `${currentDir}/${bitsDirName}/${bitName}.md`;
                    const bit = bits.get(bitPath);
                    if (bit) {
                        return bit;
                    }
                    const parentDir = path_1.resolve(currentDir, '..');
                    if (currentDir === absoluteDocsRoot ||
                        currentDir === parentDir) {
                        bail();
                    }
                    currentDir = parentDir;
                }
                function bail() {
                    throw new Error(chalk_1.red(`bit \${${bitName}} not found relative to ${path_1.relative(process.cwd(), docDir)}`));
                }
            }));
        }
    }
}
exports.replaceBits = replaceBits;
//# sourceMappingURL=replaceBits.js.map